"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
/* tslint:disable:no-submodule-imports no-console */
const _Promise = require("bluebird");
const fsExtra = require("fs-extra");
const _ = require("lodash");
const path = require("path");
const AlexaSchema_1 = require("./AlexaSchema");
const DialogflowSchema_1 = require("./DialogflowSchema");
const Spreadsheet_1 = require("./Spreadsheet");
const fs = _Promise.promisifyAll(fsExtra);
function defaultOptions(interactionOptions) {
    const rootPath = _.get(interactionOptions, "rootPath");
    const speechPath = _.get(interactionOptions, "speechPath", path.join(rootPath, "speech-assets"));
    const synonymPath = _.get(interactionOptions, "synonymPath", path.join(rootPath, "synonyms"));
    const viewsPath = _.get(interactionOptions, "viewsPath", path.join(rootPath, "app"));
    const contentPath = _.get(interactionOptions, "contentPath", path.join(rootPath, "content"));
    let platforms = _.get(interactionOptions, "platforms", ["alexa"]);
    platforms = _.isString(platforms) ? [platforms] : platforms;
    let spreadsheets = _.get(interactionOptions, "spreadsheets");
    spreadsheets = _.isString(spreadsheets) ? [spreadsheets] : spreadsheets;
    spreadsheets = spreadsheets.map((sheet) => {
        const matched = sheet.match(/docs\.google\.com\/spreadsheets\/d\/(.*)\/.*/);
        if (sheet.includes("docs.google.com/spreadsheets") && matched && _.isString(matched[1])) {
            return matched[1];
        }
        return sheet;
    });
    if (_.isEmpty(spreadsheets)) {
        throw Error("Spreadsheet were not specified in the right format");
    }
    return { rootPath, spreadsheets, speechPath, synonymPath, viewsPath, contentPath, platforms };
}
exports.buildInteraction = (interactionOptions, authKeys) => __awaiter(this, void 0, void 0, function* () {
    interactionOptions = defaultOptions(interactionOptions);
    console.time("all");
    console.time("timeframe");
    const sheets = yield Spreadsheet_1.transform(interactionOptions, authKeys);
    console.timeEnd("timeframe");
    const platforms = interactionOptions.platforms;
    const schemas = [];
    if (platforms.includes("alexa")) {
        const schema = new AlexaSchema_1.AlexaSchema(_.cloneDeep(sheets), interactionOptions);
        schemas.push(schema);
        yield fs.remove(path.join(interactionOptions.rootPath, interactionOptions.speechPath, schema.NAMESPACE));
    }
    if (platforms.includes("dialogflow")) {
        const schema = new DialogflowSchema_1.DialogflowSchema(_.cloneDeep(sheets), interactionOptions);
        schemas.push(schema);
        yield fs.remove(path.join(interactionOptions.rootPath, interactionOptions.speechPath, schema.NAMESPACE));
    }
    const fileContentsProcess = schemas
        .reduce((acc, schema, index) => {
        if (index === 0) {
            schema.buildDownloads();
            schema.buildViews();
            schema.buildViewsMapping();
            schema.buildSynonyms();
        }
        schema.invocations.map(invoc => {
            schema.build(invoc.locale, invoc.environment);
        });
        acc = acc.concat(schema.fileContent);
        return acc;
    }, [])
        .map(file => fs.outputFile(file.path, JSON.stringify(file.content, null, 2), { flag: "w" }));
    yield Promise.all(fileContentsProcess);
    console.timeEnd("all");
});
//# sourceMappingURL=InteractionBuilder.js.map