"use strict";
/// <reference types="node" />
///
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const _Promise = require("bluebird");
const google_auth_library_1 = require("google-auth-library");
const googleapis_1 = require("googleapis");
const _ = require("lodash");
const VoxaSheet_1 = require("./VoxaSheet");
function transform(options, authKeys) {
    return __awaiter(this, void 0, void 0, function* () {
        const client = google_auth_library_1.auth.fromJSON(authKeys);
        client.scopes = ["https://www.googleapis.com/auth/spreadsheets.readonly"];
        const sheets = googleapis_1.google.sheets("v4");
        const readSpreadsheet = _Promise.promisify(sheets.spreadsheets.get, { context: sheets });
        const readSheetTab = _Promise.promisify(sheets.spreadsheets.values.get, { context: sheets });
        const spreadsheetsId = options.spreadsheets;
        // TODO: error on getting spreadsheet
        let spreadsheetResp = [];
        try {
            spreadsheetResp = yield _Promise.all(spreadsheetsId.map((spreadsheetId) => readSpreadsheet({ auth: client, spreadsheetId })));
        }
        catch (e) {
            throw Error("User doesn't have access to the spreadsheet");
        }
        spreadsheetResp = _.chain(spreadsheetResp)
            .map((spreadsheet, index) => {
            const spreadsheetTitle = _.get(spreadsheet, "data.properties.title");
            const spreadsheetId = spreadsheetsId[index];
            const sheetNames = _.chain(spreadsheet)
                .get("data.sheets", [])
                .map("properties.title")
                .value();
            return sheetNames.map((sheetTitle) => {
                const voxaSheet = { spreadsheetId, spreadsheetTitle, sheetTitle, type: "none" };
                return voxaSheet;
            });
        })
            .flatten()
            .map((spreadsheet) => {
            const processorFound = _.chain(VoxaSheet_1.SheetTypes)
                .invert()
                .find((key, value) => {
                return spreadsheet.sheetTitle.indexOf(value) >= 0;
            })
                .value();
            spreadsheet.type = processorFound;
            if (processorFound) {
                return spreadsheet;
            }
            return undefined;
        })
            .compact()
            .value();
        let sheetPromises = spreadsheetResp.map((sheet) => readSheetTab({
            auth: client,
            spreadsheetId: sheet.spreadsheetId,
            range: `${sheet.sheetTitle}!A1:ZZZ`
        }));
        sheetPromises = yield _Promise.all(sheetPromises);
        return spreadsheetResp.map((sheet, index) => {
            const data = _.chain(sheetPromises[index])
                .get("data.values", [])
                .reduce(rowFormatted, [])
                .drop()
                .value();
            // Apply processor
            sheet.data = data;
            return sheet;
        });
        // global.console.log('spreadsheetResp', JSON.stringify(spreadsheetResp, null, 2));
    });
}
exports.transform = transform;
function rowFormatted(acc, next, iindex, arr) {
    const item = _.chain(arr)
        .head()
        .zip(next)
        .map((zipObj) => {
        const key = zipObj[0];
        let val = zipObj[1];
        const valTemp = _.toLower(val);
        if (_.includes(["true", "yes"], valTemp)) {
            val = true;
        }
        if (_.includes(["false", "no"], valTemp)) {
            val = false;
        }
        if (typeof valTemp === "string" && valTemp.length === 0) {
            val = undefined;
        }
        return [key, val];
    })
        .fromPairs()
        .value();
    acc.push(item);
    return acc;
}
//# sourceMappingURL=Spreadsheet.js.map